import { Connection } from './definitions';
import { genID } from './helpers';
import { createVM, VM } from './VM';

let connections: Connection[] = [];

export const createConnection = (el: HTMLIFrameElement) => {
  const instanceID = genID();

  const connection: Connection = {
    el,
    id: instanceID,
    pending: null,
    vm: null
  }

  const pendingPromise = new Promise<VM>((res, rej) => {
    function listenForSuccess(e: MessageEvent) {
      if (!!e.data.action && e.data.action === 'SDK_INIT_SUCCESS' && e.data.id === instanceID) {
        connection.vm = createVM(e.ports[0]);
        res(connection.vm);
        cleanup();
      }
    }

    function pingFrame() {
      // Ping the Iframe.
      el.contentWindow.postMessage({
        action: 'SDK_INIT',
        id: instanceID
      }, '*');
    }


    // Remove the listener and interval.
    function cleanup() {
      window.clearInterval(tryConnection);
      window.removeEventListener('message', listenForSuccess);
    }

    // First we want to set up the listener for the frame
    window.addEventListener('message', listenForSuccess);
    // Then, lets immediately ping the frame.
    pingFrame();
    // Every 500ms thereafter we'll ping until we get a response or timeout.
    // Keep track of the current try # and the max #
    const max_attempts = 20;
    let current_attempt = 0;
    const tryConnection = window.setInterval(() => {
      // If the VM connection is open, cleanup and return
      // This shouldn't ever happen, but just in case there's some race condition...
      if (!!connection.vm) {

        cleanup();
        return;
      }

      // If we've exceeded the max retries, fail this promise.
      if (current_attempt >= max_attempts) {
        cleanup();
        rej('Timeout: Unable to establish a connection with the StackBlitz VM');
        // Filter out the (now) failed connection from the connections array
        connections = connections.filter(c => c.id !== instanceID);
        return;
      }

      current_attempt++;
      pingFrame();
    }, 500);
  });

  connection.pending = pendingPromise;

  connections.push(connection);

  return connection;
}


// Accepts either the frame element OR the id.
export const getConnection = (identifier: string | HTMLIFrameElement) => {
  const key = identifier instanceof Element ? 'el' : 'id';
  const res = connections.find((c) => {
    return c[key] === identifier;
  });
  
  return !res ? null : res;
}
